
% Pour la realisation

	
inc(theme,Sem,_,[message === Val]):-
	extraire_trait(Sem,focus,Val).
inc(theme,_,_,[]).

inc(acteur,_,M,[message === Val]):-
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(acteur(Val),Traits).
inc(acteur,_,Occ,[message === Val]):-
    occurrence(Occ,Traits),
    member(acteur(Val),Traits).
inc(acteur,_,_,[]).


inc(senseur,_,M,[message === Val]):-
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(senseur(Val),Traits).
inc(senseur,_,_,[]).


inc(processus,_,M,Sem):-
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(processus(Val),Traits),
    (member(qualificateur(Q),Traits) 
             -> Sem = [message===Val,modif_verbal===Q];
                Sem = [message===Val]).
inc(processus,_,Occ,[message === Val]):-
    occurrence(Occ,Traits),
    member(processus(Val),Traits).

inc(phenomene,_,M,[message === Val]):- 
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(phenomene(Val),Traits).
inc(phenomene,_,_,[]).

inc(but,_,M,[message === Val]):-
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(but(Val),Traits).
inc(but,_,Occ,[message === Val]):-
    occurrence(Occ,Traits),
    member(but(Val),Traits).
inc(but,_,_,[]).

inc(amplitude_variation,_,M,[message === variation(X,U)]):-
    M =.. [_,Even],
    occurrence(Even,Traits),
    member(variation(X,U),Traits).


% Inclusion de la circonstance temporelle
inc(circ_temp,Sem,_,[message === Z]):- 	
        extraire_trait(Sem,loc,L),
        opt_exp_loc(L,Z).


% On reecrit l'expression de la localisation avant de l'utiliser comme
% semantique

opt_exp_loc(X,X):-
           X=duree(_,_);
	   X=occurrence(_).
opt_exp_loc([A,B,C],[A,B,C]).
opt_exp_loc(incldans([_,minute,M],[_,heure,H]),[ctx,minute,horaire(H,M)]).
opt_exp_loc(incldans([_,minute,M],incldans([_,heure,H],Z)),
            incldans([ctx,minute,horaire(H,M)],Z2)):-
	opt_exp_loc(Z,Z2).
opt_exp_loc(Exp1,Exp2):-
	Exp1 =.. [Rel,X,Z],
        member(Rel,[incl,incldans,deb,term,debpar,termpar]),
        opt_exp_loc(Z,Z2),
        Exp2 =.. [Rel,X,Z2].
opt_exp_loc(pos_rel(N,X,Z),pos_rel(N,X,Z2)):-
        opt_exp_loc(Z,Z2).
opt_exp_loc(Exp1,Exp2):-
	Exp1 =.. [Rel,X,Y,Z],
        member(Rel,[entre,etendue]),
        opt_exp_loc(Y,Y2),
        opt_exp_loc(Z,Z2),
        Exp2 =.. [Rel,X,Y2,Z2].
opt_exp_loc(Exp1,Exp2):-
	Exp1 =.. [Rel,X,Y,Dec],
        member(Rel,[avant,apres]),
        opt_exp_loc(X,X2),
        opt_exp_loc(Y,Y2),
        Exp2 =.. [Rel,X2,Y2,Dec].


% Inclusion de la tete lors de la realisation d'un groupe nominal
% Cas ou le message de la phrase ne concerne qu'un objet
% Aucune occurrence n'est relatee
inc(tete,Sem,_,Sem).

inc(det,Sem,_,Sem).


inc(identificateur,_,_,Sem):-
	val_sem(tete,Sem).


inc(specificateur,Sem,_,Sem).



inc(interieur_parenthese,Sem,_,Sem).


inc(id_ref,_,Objet,[message === Val]):-
        desc(Objet,P),
	member(id(Val),P).
inc(onto_ref,_,Objet,[message === O]):-
        desc(Objet,P),
        member(onto(O),P).
inc(onto_ref,Sem,_,Sem).


% Inclusion d'un quantificateur dans un groupe nominal
inc(quantificateur,_,duree(Q,_),[message === Q]).
inc(quantificateur,Sem,_,[message === Q]):-
	extraire_trait(Sem,quantite,Q).


% Inclusion d'un quantificateur de duree
% (Qui specifie le nombre d'intervalles unitaires composant la
% duree)
inc(quant_duree,_,duree(Q,U),[message===U,quantite===Q]).
	


% Inclusion d'un positionneur d'un adverbial temporel
inc(positionneur,Sem,_,Sem).


% Inclusion de la zone de reference temporelle pour un adverbial temporel
% relationnel
inc(zone_ref_temp,_,X,[message === Z]):-
        id_temps_ref(X,Z).
inc(zone_ref_temp,_,duree(Q,U),[message === duree(Q,U)]).


% Inclusion des ancrages anterieurs et posterieurs pour une duree
% doublement bornee

inc(ancrage_ante,_,X,[message===A]):-
        id_temps_ref1(X,A).
inc(ancrage_post,_,X,[message===A]):-
        id_temps_ref2(X,A).







% Inclusion du localisateur temporel dans le cas d'une designation 
% directe

% La zone est un temps qui en inclut un autre et n'a pas de denomination
% On ne connait que son unite
inc(des_zone,_,incl([_,U,N],_),[message===U]):-
        var(N). 
% La zone est une constante et sa denomination n'est pas un nombre
inc(des_zone,_,Exp,[message===Denom]):-
	id_temps_designe(Exp,[_,_,Denom]),
        nonvar(Denom),
        \+ number(Denom). 
% La zone est un moment (anaphorique)
inc(des_zone,_,[_,U,N],[message===moment]):-
	var(U),var(N).
% La zone est une annee dont on connait le numero
inc(des_zone,_,[_,annee,N],[message===N]):-
        number(N).
% La zone est une constante avec denomination numerique
inc(des_zone,_,[_,U,N],[message===U,position===N]):-
        number(N).
% La zone est une constante incluse dans une autre
inc(des_zone,_,incldans([_,U,N],_),[message===U,position===N]).
% Cas special des heures minutes   (comme 8h10)
inc(des_gzone,_,Exp,[message===horaire(H,M)]):-
	id_temps_designe(Exp,[_,minute,horaire(H,M)]).
% Inclusion d'une occurrence qui sert de localisateur temporel
inc(des_zone,_,occurrence(O),[loc===occurrence(O),enon===E,message === O]):-
        val_sem(racine..prop,PropSem),
        extraire_trait(PropSem,enon,E).

inc(des_zone,Sem,_,Sem).


inc(decalage_temporel,_,pos_rel(X,[_,U,_],_),[message === U,quantite===Q]):-
        abs(X,Q).
inc(decalage_temporel,_,Exp,[message === U,quantite===X]):-
        member(Exp,[avant(_,_,duree(X,U)),apres(_,_,duree(X,U))]).



inc(modif_verbal,Sem,_,[message===A]):-
	extraire_trait(Sem,modif_verbal,A).








inc(valeur_variation,_,variation(X,U),[message===U,quantite===X]).

inc(valeur_init_variation,_,variation(X-_,U),[message===U,quantite===X]).
inc(valeur_finale_variation,_,variation(_-X,U),[message===U,quantite===X]).



% Inclustion d'un complement de nom
inc(comp_nom,_,X,[message=== C]):-
         desc(X,D),
         member(de(C),D).    	
 



inc(mot_adj,Sem,_,[message===A]):-
         extraire_trait(Sem,position,A).




% Pour les tests
propriete(P,Valeur):-
        valeur(constituant,C),
        extraire_trait(C,message,M),        
        prop(P,C,M,Valeur),!.
propriete(F,P,Valeur):-
        val_sem(F,V),
        extraire_trait(V,message,M),        
	prop(P,V,M,Valeur),!.



% Test d'informations liees a la metafonction interpersonnelle
prop(interp,Sem,_,commande):-
        extraire_trait(Sem,type_comm,commande).


% Test si deux fonction ont la meme valeur au trait "valeur"
prop(meme_ref(F1,F2),_,_,oui):-
    val_sem(F1,SemF1),
    val_sem(F2,SemF2),
    extraire_trait(SemF1,message,V),
    extraire_trait(SemF2,message,V).
prop(meme_ref(F1,F2),_,_,non):-
    val_sem(F1,SemF1),
    val_sem(F2,SemF2),
    extraire_trait(SemF1,message,V1),
    extraire_trait(SemF2,message,V2),
    V1 \== V2.
    
% Deux chemins de fonctions designent exactement le meme constituant
prop(id(F1,F2),_,_,oui):-
    val_const(F1,const(Id,_,_,_)),
    val_const(F2,const(Id,_,_,_)).
prop(id(F1,F2),_,_,non):-
    val_const(F1,const(Id1,_,_,_)),
    val_const(F2,const(Id2,_,_,_)),
    Id1 \== Id2.


% Determine le type d'ancrage
prop(ancrage,_,duree(_,_),unique).
prop(ancrage,_,occurrence(_),unique).
prop(ancrage,_,[_,_,_],unique).
prop(ancrage,_,Exp,unique):-
    functor(Exp,Rel,_),
    member(Rel,[incldans,incl,pos_rel,termpar,debpar,deb,term,avant,apres]).
prop(ancrage,_,Exp,double):-
    functor(Exp,Rel,_),
    member(Rel,[etendue,entre]).



% Donne le ou les ancrages d'une constante
prop(ancrage_unique,SemZone,V,A):-
    prop_ancrage_unique(V,A).
prop(ancrage1,_,V,A):-
    id_temps_ref1(V,Ref1),
    prop_ancrage_unique(Ref1,A).
prop(ancrage2,_,V,A):-
    id_temps_ref2(V,Ref2),
    prop_ancrage_unique(Ref2,A).

prop_ancrage_unique(occurrence(A),A):-!.
prop_ancrage_unique([C,U,N],[C,U,N]).
prop_ancrage_unique(Exp,A):-
    member(Exp,[incldans(_,X),incl(_,X),pos_rel(_,_,X),avant(_,X,_),
                apres(_,X,_),debpar(_,X),termpar(_,X),term(X,_),deb(X,_)]),
    prop_ancrage_unique(X,A).



% On determine ici si le contenu semantique est une occurrence
% a rapporter ou un objet a designer
% Appele par CONSTITUANT
prop(type_message,_,Objet,Objet):-
        desc(Objet,_).
prop(type_message,_,O,objet):-
        desc(O,_).
prop(type_message,_,evenement(_),even_simple).
prop(type_message,_,situation_ouverte(_),even_simple).
prop(type_message,_,situation_resultante(_),even_simple).
prop(type_message,_,Exp,localisateur):-
        id_temps_designe(Exp,[_,_,_]).



prop(type_processus,_,_,P):-
        val_sem(processus,SemProcessus),
        extraire_trait(SemProcessus,message,X),
	lexique(verbe,X,_,Info),
	member(processus===P,Info).

% Determine le genre d'un objet

prop(genre,_,V,G):-
        desc(V,Desc),
	member(onto(O),Desc),
        (member(O,[pays,province]), \+ member(id(_),Desc);
         not(member(O,[pays,province]))),
        lexique(nom,O,_,Info),
	member(genre===G,Info).
prop(genre,_,V,G):-
        desc(V,Desc),
	(member(onto(pays),Desc);
 	 member(onto(province),Desc)),
        member(id(IdPays),Desc),
	lexique(nom,IdPays,_,Info),
	member(genre===G,Info).
prop(genre,_,Nom,G):-
	lexique(nom,Nom,_,Info),
        member(genre===G,Info).


% Determine le nombre d'un objet
prop(nombre,_,X,pluriel):-
        desc(X,D),
	member(quantite(N),D),
        (N=pluriel; number(N),N>1 ).
prop(nombre,Sem,_,pluriel):-
	extraire_trait(Sem,quantite,Q),
	Q >= 2.




% Determine si un verbe est modifie par un adverbe
prop(modif_verbal,Sem,_,X):-
	extraire_trait(Sem,modif_verbal,X).


% Determine la nature ontologique d'un objet
prop(type_objet_refere,_,V,Type):-
    desc(V,Desc),
    member(onto(Type),Desc).


      

% Determine si un objet est inclu dans le contexte
prop(contexte(X),_,V,oui):-
    val_sem(prop,SemProp),
    member(contexte===Cont,SemProp),
    member(X===L,Cont),
    member(V,L).
prop(contexte(X),S,V,non):-
    \+ prop(contexte(X),S,V,oui).

prop(trait_semantique(X),Sem,_,Valeur):-
    extraire_trait(Sem,X,Valeur).



% Specifie le type de zone temporelle
% Appele par des_zone_temp, aspect_zone, ancrage_duree
prop(type_zone,_,V,T):-
    prop_type_zone(V,T).

prop_type_zone(X,constante):-
    member(X,[[_,_,_],incldans(_,_),incl(_,_),pos_rel(_,_,_),duree(_,_)]).
prop_type_zone(X,duree):-
    functor(X,Rel,_),
    member(Rel,[etendue,duree,termpar,debpar]).
prop_type_zone(X,duree_bornee):-
    functor(X,Rel,_),
    member(Rel,[etendue,termpar,debpar]).
prop_type_zone(X,duree_quantifiee):-
    functor(X,Rel,_),
    member(Rel,[duree]).
prop_type_zone(occurrence(_),occurrence).
prop_type_zone([_,_,_],position).
prop_type_zone(incldans(_,_),position).
prop_type_zone(horaire(_,_),horaire).

  


prop(denomination,_,Exp,directe):-
    id_temps_designe(Exp,[_,_,horaire(_,_)]).
prop(denomination,_,Exp,directe):-
    id_temps_designe(Exp,[_,_,X]),
    \+ number(X).
prop(denomination,_,Exp,directe):-
    id_temps_designe(Exp,[_,annee,X]),
    number(X).
prop(denomination,_,Exp,pos_dans_ordre):-
    id_temps_designe(Exp,[_,_,X]),
    number(X).





% Identification du type de relation ponctuelle
% Appele par LOCALISATION

prop(type_rel,_,Exp,Type):-
    prop_type_rel(Exp,Type).

prop_type_rel(pos_rel(_,_,_),decalage).
prop_type_rel(avant(_,_,_),avant).
prop_type_rel(apres(_,_,_),apres).
prop_type_rel(etendue(_,_,_),etendue).
prop_type_rel(entre(_,_,_),entre).
prop_type_rel(termpar(_,_),termpar).
prop_type_rel(debpar(_,_),termpar).
prop_type_rel(term(_,_),termpar).
prop_type_rel(deb(_,_),termpar).
prop_type_rel(duree(_,_),duree).
prop_type_rel(incl(_,_),incl).
prop_type_rel(incldans(_,_),incldans).



% Specifie la fermeture d'une duree
% Appele par ANCRAGE_DUREE
prop(fermeture,_,V,F):-
    prop_fermeture(V,F).

prop_fermeture(termpar(_,_),post).
prop_fermeture(debpar(_,_),ante).
prop_fermeture(etendue(_,_,_),double).


% Indique si le localisateur est simple ou compose
prop(complexite_loc,_,[_,_,_],simple).
prop(complexite_loc,_,X,composee):-
    X\==[_,_,_].


% Indique si le temps de reference est contextuelle

prop(cadre_contextuel,_,etendue(_,X,Y),oui):-
    id_temps_designe(X,[_,_,N]),
    id_temps_designe(Y,[_,_,N]).
prop(cadre_contextuel,_,_,non).



% Precise le sens d'une relation temporelle
% Appele par SENS_RELATION

prop(sens_relation,_,V,S):-
    prop_sens_relation(V,S).
prop_sens_relation(avant(_,_,_),anteriorite).
prop_sens_relation(pos_rel(X,_,_),anteriorite):-
    X < 0.
prop_sens_relation(apres(_,_,_),posteriorite).
prop_sens_relation(pos_rel(X,_,_),posteriorite):-
    X > 0.



% Precise le nombre d'unite dans une relation 
% d'anteriorite/posteriorite

prop(decalage_temporel,_,V,D):-
    prop_mesure_rel(V,D).

prop_mesure_rel(pos_rel(X,_,_),Dec):-
    abs(X,Dec).
prop_mesure_rel(Zone,indefini):-
    member(Zone,[avant(_,_,indefini),apres(_,_,indefini)]).
prop_mesure_rel(Zone,Dec):-
    member(Zone,[avant(_,_,duree(Dec,_)),apres(_,_,duree(Dec,_))]).

% Retourne l'unite de temps du cadre temporel d'un localisateur.
prop(unite,_,V,Unite):-
    id_temps_designe(V,[_,U,_]),
    nonvar(U),
    Unite=U.
prop(unite,_,_,indefini).
    


% Indique si le localisateur Zone designe une relation par
% rapport a une autre zone.
prop(des_int_temp,_,V,D):-
    prop_des_int_temp(V,D).
prop_des_int_temp(Zone,relationnelle):-
    functor(Zone,Op,_),
    member(Op,[pos_rel,avant,apres,termpar,debpar]).




% Retourne la valeur d'un des reperes suivants:
% loc, enon, persp, ref
prop(repere(R),_,_,Val):- 
    val_sem(prop,Sem),
    extraire_trait(Sem,R,Val).


% Exprime la relation entre deux reperes.
% Choix possibles de reperes: enon loct persp ref even
% Appele par le systeme localisation

prop(relation(X,Y),_,_,Rel):-
       identifier_op_rel_temp(X,IdX),
       identifier_op_rel_temp(Y,IdY),
       prop_relation(IdX,IdY,Rel).
       
identifier_op_rel_temp(even,Id):-
       val_sem(prop,Sem),
       extraire_trait(Sem,message,Obj),
       argument(1,Obj,Id).     
identifier_op_rel_temp(X,Id):-
       member(X,[ref,enon,persp]),
       val_sem(prop,Sem),
       extraire_trait(Sem,X,Obj),
       argument(1,Obj,Id).
identifier_op_rel_temp(loc,Id):-
       val_sem(prop,Sem),
       extraire_trait(Sem,loc,Exp),   
       id_temps_designe(Exp,[Id,_,_]). 
identifier_op_rel_temp(loc,Id):-
       val_sem(prop,Sem),
       extraire_trait(Sem,loc,occurrence(Id)).   
identifier_op_rel_temp(zone,Id):-
       val_sem(constituant,Sem),
       extraire_trait(Sem,message,Exp),
       id_temps_designe(Exp,[Id,_,_]).    
identifier_op_rel_temp(ancrage,Id):-
       propriete(constituant,ancrage_unique,Exp),
       id_temps_designe(Exp,[Id,_,_]).           
identifier_op_rel_temp(A,Id):-
       member(A,[ancrage1,ancrage2]),
       propriete(constituant,A,Exp),
       id_temps_designe(Exp,[Id,_,_]). 

prop_relation(X,Y,not(Rel)):-
    \+ prop_relation(X,Y,Rel).

prop_relation(X,Y,simultane):-
    les_relations(Rel),
    simultane(Rel,X,Y).
prop_relation(X,Y,avant):-
    les_relations(Rel),
    not simultane(Rel,X,Y),
    ferm_trans(Rel,[X],X,Y,avant).
 
prop_relation(X,Y,incldans):-
    les_relations(Rel),
    member(incldans(X,Y),Rel).

prop_relation(X,Y,apres):-
    prop_relation(Y,X,avant).


ferm_trans(Rel,_,X,Y,avant):-
    member(avant(X,Y),Rel);
    member(equ(X,Y),Rel);
    member(equ(Y,X),Rel);
    member(incldans(Y,X),Rel).

ferm_trans(Rel,_,X,Y,avant):-
     id_objet_interm(Rel,X,_,Z,TypeRel),
     member(TypeRel,[incldans,term]),
     prop_relation(Z,Y,avant).
ferm_trans(Rel,Visites,X,Y,avant):-
     id_objet_interm(Rel,X,Visites,Z,noninclus),
     ferm_trans(Rel,Visites,Z,Y,avant).

id_objet_interm(Rel,X,Visites,Z,incldans):-
    member(incldans(X,Z),Rel),
    not member(Z,Visites).
id_objet_interm(Rel,X,Visites,Z,term):-
    member(incldans(X,Z),Rel),
    not member(Z,Visites).
id_objet_interm(Rel,X,Visites,Z,noninclus):-
    (member(equ(X,Z),Rel);member(equ(Z,X),Rel);member(avant(X,Z),Rel)),
    not member(Z,Visites).


prop_relation(X,X,equ).
prop_relation(X,Y,equ):-
        les_relations(Rel),
	(member(equ(Y,X),Rel);
	 member(equ(X,Y),Rel)).
prop_relation(X,Y,incldans):-
        les_relations(Rel),
	 (member(incldans(X,Y),Rel);
	  member(incl(Y,X),Rel)).
prop_relation(incldans(constante(Y)),Y,incldans).
      


prop_relation(X,Y,incl):-
        les_relations(Rel),
	 (member(incl(X,Y),Rel);
	  member(incldans(Y,X),Rel)).
prop_relation(X,Y,term):-
        les_relations(Rel),
          (member(term(X,Y),Rel);
	   member(termpar(Y,X),Rel)).
prop_relation(X,Y,termpar):-
        les_relations(Rel),
          (member(termpar(X,Y),Rel);
	   member(term(Y,X),Rel)).
prop_relation(X,Y,deb):-
        les_relations(Rel),
          (member(deb(X,Y),Rel);
	   member(debi(Y,X),Rel)).
prop_relation(X,Y,debpar):-
        les_relations(Rel),
          (member(debpar(X,Y),Rel);
	   member(deb(Y,X),Rel)).

% Pour determiner le type d'auxiliaire d'un verbe.
% Appele par AUXILIAIRE
prop(composition,SemV,X,auxiliaire(A)):-
	lexique(verbe,X,_,Info),
	member(auxiliaire===A,Info).

% Appele par TYPE_NOM
prop(propre,_,N,oui):-
        desc(N,D),
	member(id(_),D).

% Determine quel est le role joue par l'objet dans 
% la communication
% Appele par les systemes pronom et verbal
prop(role_comm,_,V,Role):-
    prop_role_comm(V,Role).
prop_role_comm(X,locuteur):-
	val_sem(prop,Mess),
        extraire_trait(Mess,locuteur,X).
prop_role_comm(X,interlocuter):-
	val_sem(prop,Mess),
        extraire_trait(Mess,interlocuteur,X).
prop_role_comm(_,tiers).


% (Comme un nom propre)
% Appelee par le systeme nominal
prop(identificateur,_,V,I):-
        desc(V,D),
        member(id(I),D).
prop(connu,Tete,V,oui):-
	desc(V,D),
        member(connu,D).
prop(connu,Tete,_,non).


% Appelee par le systeme proposition
prop(aspect,_,Mess,situation):-
	 (Mess = situation_ouverte(_);
	  Mess = situation_resultante(_)).
prop(aspect,_,_,evenement).

% Appelee par le systeme proposition
prop(type_situation,_,Mess,resultante):-
        Mess = situation_resultante(_).
prop(type_situation,_,_,ouverte).


% Appelee par le systeme verbal
prop(prolongation_verbe,_,X,Type):-
    lexique(verbe,X,_,Info),
    member(requis===Type,Info).



% Indique si la localisation temporelle est 
% specifiee dans la circonstance.
% Appelee par le systeme circ_temporelle
prop(loc_temporelle,Sem,_,specifiee):-
        extraire_trait(Sem,loc,L),
        L \== nil.
prop(loc_temporelle,_,_,non_specifiee).




prop(complexite,_,S,compose):-
	desc(S,L),
	member(de(_),L).
prop(complexite,_,_,simple).

prop(complementation,_,S,attributif):-
	desc(S,L),
	member(de(_),L).
prop(complementation,_,_,autre).  



prop(type_variation,_,M,bornee) :-
	M=variation(_-_,_).
prop(type_variation,_,M,delta) :-
	M=variation(X,_),
        \+ (X=_-_).



% Autres fonctions

idmessage(Id):-
	message(Id).

% Decompose un localisateur temporel.
% Retourne l'operateur dans Op et la premiere operande dans O1.
% S'il y a une deuxieme operande, elle sera retournee dans O2.
idopertemp(Zone,Operateur,Operande1,Operande2):-
     functor(Zone,Operateur,_),
    (member(Operateur,[avant,apres,incldans,incl,
                       deb,debpar,term,termpar]) ->
     Zone =.. [Operateur,Operande1|_];
     Zone =.. [Operateur,Operande1,Operande2|_]).


% Determine si Z est un objet temporel
objet_temporel(Z):- constante(Z,_).
objet_temporel(Z):- occurrence(Z,_).
