% UTIL.PL
% Contient des utilitaires:
%                  abs
%                  ajouter_fin
%                  concatener
%                  concatliste
%                  deux_chiffres
%                  enleve
%                  enleve_liste
%         X         finit_par_voyelle
%                  fusionner
%                  impair
%                  inclus_liste
%                  liste
%                  long_liste
%                  long_string
%                  merge
%                  moinsmod
%                  mon_concat
%                  nieme
%                  nom
%                  nom1
%                  pair
%                  plusmod
%                  remplace
%                  renverse
%                  retirer_dernier
%                  retirer_derniere_lettre
%                  t
%                  trouve_dernier
%                  trouve_precedent
%                  trouve_suivant
%                  unit



%Inverse tous les elements d'une liste
renverse([],[]).
renverse([X|Reste],ListeRenv):-
	renverse(Reste,ResteRenv),
	append(ResteRenv,[X],ListeRenv).
     
% Enleve un element d'une liste
enleve(X,[X|Reste],Reste).
enleve(X,[Z|Reste],[Z|Resultat]):- enleve(X,Reste,Resultat).
enleve(_,[],[]).

% Enleve plusieurs elements d'une liste
enleve_liste([],Liste,Liste).
enleve_liste([X|Reste],Liste_init,Liste_res):-
	enleve(X,Liste_init,Nouv_liste),
	enleve_liste(Reste,Nouv_liste,Liste_res).

% Ajouter un element a la fin d'une liste
ajouter_fin(E,[],[E]).
ajouter_fin(E,[X|R],[X|Z]):- ajouter_fin(E,R,Z).


% Retire le dernier element d'une liste
retirer_dernier([],[]).
retirer_dernier([_],[]).
retirer_dernier([X|Xs],[X|Ys]):-
        retirer_dernier(Xs,Ys).


% Remplace un element d'une liste par un autre.
remplace(_,_,[],[]).
remplace(Ancien,Nouv,[Ancien|AncListe],[Nouv|AncListe]).
remplace(Ancien,Nouv,[X|AncListe],[X|ListeMod]):-
	remplace(Ancien,Nouv,AncListe,ListeMod).



% concatenation de deux listes
merge([],[]).
merge([X|Xs],R):-
    merge(Xs,Xm),
    append(X,Xm,R).




% Fusionne une serie de listes
fusionner([],[]).
fusionner([L1|Reste],[L1|Resultat]):-
        \+ liste(L1),
        fusionner(Reste,Resultat).
fusionner([L1|Reste],Resultat):-
	fusionner(Reste,ResInt),
	append(L1,ResInt,Resultat).


% Trouve dans une liste l'element qui suit l'element donne en parametre
trouve_suivant([_],_,nil).
trouve_suivant([No,Suivant|_],No,Suivant).
trouve_suivant([_|Reste],No,Suivant):-
	trouve_suivant(Reste,No,Suivant).

% Trouve dans une liste l'element qui precede l'element donne en parametre
trouve_precedent([_],_,nil).
trouve_precedent([Precedent,No|_],No,Precedent).
trouve_precedent([_|Reste],No,Precedent):-
	trouve_precedent(Reste,No,Precedent).

% Trouve le dernier element d'une liste
trouve_dernier([Dernier],Dernier).
trouve_dernier([_|Xs],Dernier):-
	trouve_dernier(Xs,Dernier).



% Verifie si une structure est une liste
liste(X):-
    var(X),!,
    fail.
liste([]).
liste([_|_]).


% Retourne le nombre d'elements dans une liste
long_liste([],0).
long_liste([_|Reste],Long):-
     long_liste(Reste,LongReste),
     Long is LongReste + 1.

% Verifie si une liste est incluse dans une autre
inclus_liste([],_).
inclus_liste([X|Reste],Liste):-
     member(X,Liste),
     inclus_liste(Reste,Liste).



nieme(_,[Dernier],Dernier).
nieme(1,[Elem|_],Elem).
nieme(N,[_|Autres],Elem):-
     NN is N - 1,
     nieme(NN,Autres,Elem).


% Concatener deux chaines de caracteres
concatener(Chaine1,Chaine2,Res):-
    nom(Chaine1,Ch1),
    nom(Chaine2,Ch2),
    append(Ch1,Ch2,ChRes),
    nom(Res,ChRes).

% Concatener une liste d'atomes

concatliste(Liste,LisCon):- ccl(Liste,'',LisCon).
ccl([],X,X).
ccl([X|Reste],In1,ResFin):-
    concat(In1,X,In2),
    ccl(Reste,In2,ResFin).


% Concatener deux mots en mettant un espace entre eux
unit(Mot1,Mot2,Resultat):-
    concatener(Mot1,' ',MotEsp),
    concatener(MotEsp,Mot2,Resultat).


% Retourne la longueur d'une chaine de caracteres
long_string(Chaine,Long):-
   nom(Chaine,ListeChaine),
   long_liste(ListeChaine,Long).



abs(X,X):-X>0.
abs(X,Y):-X<0,Y is -1*X.


nom(X,Xl) :-				% explose
    \+ var(X),!, name(X,Codes), nom1(Codes,Xl).
nom(X,Xl) :-				% compose
    \+ var(Xl),!, nom1(Codes,Xl), name(X,Codes).
nom(X,X1) :-
    \+ var(X), \+var(X1),
    nom(X2,X1),
    X == X2.

nom1([],[]).
nom1([X|Xs],[X1|X1s]):- name(X1,[X]), nom1(Xs,X1s).






% OPERATIONS AVEC MODULO

plusmod(X,Y,Mod,Z):-
    X1 is X mod Mod,
    Y1 is Y mod Mod,
    Z is (X1+Y1) mod Mod.
moinsmod(X,Y,Mod,Z):-
    X1 is X mod Mod,
    Y1 is Y mod Mod,
    Z1 is (X1-Y1) mod Mod,
    (Z1 < 0 -> Z is Mod+Z1; Z is Z1).



% VERIFICATION DE LA PARITE D'UN NOMBRE

pair(X):-     0 is X mod 2.
impair(X):-   1 is X mod 2.







finit_par_voyelle(X):-
    nom(X,N),
    fpv(N).

fpv([X]):-voyelle(X).
fpv([_|Z]):-fpv(Z).

finit_par_lettre(L,X):-
    nom(X,N),
    fpl(L,N).

fpl(L,[L]).
fpl(L,[_|Z]):-fpl(L,Z).



retirer_derniere_lettre(X,Y):-
    nom(X,X1),
    retirer_dernier(X1,X2),
    nom(Y,X2).




deux_chiffres(M,M):- nom(M,[_,_]).
deux_chiffres(M,M1):- nom(M,[X]),concat('0',M,M1).




t :- trace.



mon_concat(X,Y,Z):-
	nom(X,NomX),
	nom(Y,NomY),
	append(NomX,NomY,NomZ),
	nom(Z,NomZ).
