% Liste de clause qui servent autant a l'analyse qu'a la generation



% Recherche dans la liste d'associations le temps associe
% a une constante temporelle.
% Si le temps n'existe pas, on cree une constante et on 
% ajoute le binding dans la liste d'association
binding(Const,Temps,Assoc,Assoc):-
  member((Const<==>Temps),Assoc),!.
binding(Constante,Temps,Ain,[Constante<==>Temps|Ain]):-
  new_term(t,Constante).
  

% Mise a jour du contexte

% L'instant est celui du top du contexte
update_cont((_<==>I),[(T<==>I)|Cont],[(T<==>I)|Cont],A,A):-!.  
% L'instant est ajoute directement au top du contexte
% (Il est donc inclus dans cet instant au top)
update_cont((T<==>I),[(T2<==>I2)|Cont],[(T<==>I),(T2<==>I2)|Cont],A,A):-
    % I2 peut constituer un cadre direct pour I
    type(I,U),
    def_cadre(_,cadre(I2,U,Dlist)),
    element(I,Dlist),!.
% L'instant est du meme type que celui sur le top du contexte
% Mais situe avant ou apres. L'instant englobant celui
% du top demeure englobe aussi celui a ajouter au contexte.
% Dans ce cas le nouvel instant remplace celui du top.
update_cont((T<==>I),[_<==>I2,T3<==>I3|Cont],[T<==>I,T3<==>I3|Cont],A,A):-
    type(I2,Type),
    type(I,Type),
    cadre_sup(I,cadre(I3,Type,_),[]),!.
% L'instant est inclus dans un instant du meme type que celui
% au top du contexte, mais qui est situe avant ou apres.
update_cont((T<==>I),[_<==>I2,T3<==>I3|Cont],[(T<==>I),(Tnew<==>I4),T3<==>I3|Cont],A,[Tnew<==>I4|A]):-
    type(I2,Type),
    projection(I,cadre(I3,Type,_),I4,[]),
    new_term(t,Tnew),!.
% L'instant n'est pas inclus ni dans celui au top ni dans un instant
% du meme type avant ou apres. Il faut donc eliminer le top et
% tester avec la suite.
update_cont((T<==>I),[_|Cont],NewCont,Ain,Aout):-
    update_cont((T<==>I),Cont,NewCont,Ain,Aout).





% Selectionne dans le contexte, l'instant de reference pour 
% celui qu'on veut identifier

% Select_cont retourne le premier 
% instant qui peut constituer un cadre pour l'instant a identifier 
select_cont(_,U,[(Tdrs<==>Tkrs)|_],(Tdrs<==>Tkrs)):-
    % Top peut constituer un cadre direct pour T
    def_cadre(_,cadre(Tkrs,U,_)).
select_cont(T,U,[_|Reste],(Tdrs<==>Tkrs)):-
    select_cont(T,U,Reste,(Tdrs<==>Tkrs)).







id_temps_designe([X,Y,Z],[X,Y,Z]).
id_temps_designe(incldans(X,_),R):-id_temps_designe(X,R).
id_temps_designe(incl(X,_),R):-id_temps_designe(X,R).
id_temps_designe(pos_rel(_,X,_),R):-id_temps_designe(X,R).
id_temps_designe(avant(X,_),R):-id_temps_designe(X,R).
id_temps_designe(apres(X,_),R):-id_temps_designe(X,R).
id_temps_designe(term(X,_),R):-id_temps_designe(X,R).
id_temps_designe(deb(X,_),R):-id_temps_designe(X,R).
id_temps_designe(termpar(X,_),R):-id_temps_designe(X,R).
id_temps_designe(debpar(X,_),R):-id_temps_designe(X,R).
id_temps_designe(entre(X,_,_),R):-id_temps_designe(X,R).
id_temps_designe(etendue(X,_,_),R):-id_temps_designe(X,R).



id_temps_ref([X,Y,Z],[X,Y,Z]).
id_temps_ref(pos_rel(_,_,X),X).
id_temps_ref(avant(_,X,_),X).
id_temps_ref(apres(_,X,_),X).
id_temps_ref(Exp,X):-
   Exp =.. [Rel,_,X],
   member(Rel,[incldans,incl,term,termpar,deb,debpar]).
id_temps_ref1(Exp,X):-
   Exp =.. [Rel,_,X,_],
   member(Rel,[entre,etendue]).
id_temps_ref2(Exp,X):-
   Exp =.. [Rel,_,_,X],
   member(Rel,[entre,etendue]).
