


% Pour identifier un instant a partir d'un nom N.
% On recherche un cadre C tel que 

%     - l'instant est element de la D-liste de ce cadre C
%     - Soit F une fonction de denommination associee au
%       cadre C, on a F(I) =  N.     

% Il s'agit de l'instant d'enonciation
identifier([n,_,_],ContIn,ContOut,Assoc,Assoc,(n<==>N)):-
    enonciation(N),
    update_cont((n<==>N),ContIn,ContOut,_,_),!.
% L'instant est deja connu
% (Il est donc present dans la liste d'association)
identifier([T,_,_],ContIn,ContOut,Assoc,AssocOut,(T<==>I)):-
    nonvar(T),
    binding(T,I,Assoc,_),
    update_cont((T<==>I),ContIn,ContOut,Assoc,AssocOut),!.

% L'instant peut etre identifie directement
% T est associe a un instant I tel que cadre(C,U,Dlist) 
% C est un instant dans le contexte et I est element de Dlist
identifier([T,U,N],ContIn,NewCont,AssocIn,[T<==>I|AssocOut],(T<==>I)):-
    select_cont(T,U,ContIn,(_<==>I2)),
    % Cas ou on cherche I
       (var(I),identifier2(I2,U,N,I);
    % Cas ou on a deja la valeur de I (on ne veux que verifier
    %  que I est un instant pouvant etre denomme N et inclus dans I2)
        nonvar(I),identifier3(I2,U,N,I)),
    update_cont((T<==>I),ContIn,NewCont,AssocIn,AssocOut),!.


% L'instant est inclus dans un autre
% T est associe a un instant I tel que cadre(B,U,Dlist) et I est
% element de Dlist. 
identifier(incldans([T,U,N],B),ContIn,NewCont,
           AssocIn,[T<==>I|AssocOut],(T<==>I)):-
    identifier(B,ContIn,ContInter,AssocIn,AssocInter,(_<==>I2)),
    % Cas ou on cherche I
    (var(I),identifier2(I2,U,N,I);
    % Cas ou on a deja la valeur de I (on ne veux que verifier
    %  que I est un instant pouvant etre denomme N et inclus dans I2)
    nonvar(I),identifier3(I2,U,N,I)),
    update_cont((T<==>I),ContInter,NewCont,AssocInter,AssocOut),!.
% L'instant est inclus dans un autre
% Cas ou la relation d'inclusion doit etre traitee indirectement
identifier(incldans([T,U,N],B),ContIn,ContOut,
           AssocIn,AssocOut,(T<==>I)):-
    identifier(B,ContIn,ContInter,AssocIn,AssocOut,(T2<==>I2)),
    % Cas ou on cherche I
    (var(I),
    binding(Tere,ere1(1),AssocIn,_),
    identifier(filtre(U,[T,U,N],[Tere,_,_]),ContInter,_,AssocOut,_,(_<==>I3)),
    def_cadre(_,cadre(I3,U,DlistI3)),
    chercher_incldans(DlistI3,I2,I),
    update_cont(T<==>I,ContInter,ContOut,_,_)).


chercher_incldans([X|_],Y,X):- inclus(X,Y),!.
chercher_incldans([_|Xs],Y,I):- chercher_incldans(Xs,Y,I).


% L'instant inclut un autre instant
% T est associe a un instant I tel que cadre(I,U,Dlist) et B est element
% de Dlist
identifier(incl([T,U,_],B),ContIn,NewCont,AssocIn,[T<==>I|AssocOut],(T<==>I)):-
    identifier(B,ContIn,ContInter,AssocIn,AssocInter,(_<==>I2)),
    (type(I2,U) ->
             % Si T et I2 ont la meme unite c'est I2 qu'on retourne
             I = I2;
             % Sinon on retourne un referent d'un cadre superieur
             cadres_englobants(I2,C,[]), 
             member((cadre(I,_,_),_),C)),
             type(I,U),
    update_cont((T<==>I),ContInter,NewCont,AssocInter,AssocOut),!.


% On cherche un instant dont la position est donnee relativement a
% Un instant de reference B.
% La constante T sera associee au (Val)ieme instant de type U denomme N
% en calculant a partir de B
identifier(pos_rel(Val,A,B),ContIn,ContOut,
           AssocIn,AssocOut,Link):-
    identifier(B,ContIn,_,AssocIn,AssocInter,(_<==>I2)),
    type(I2,U),
    def_cadre(_,cadre(ere1(1),U,Dliste)),
    ieme(Pos,Dliste,I2),
    chercher_le_bon(Val,Pos,Dliste,A,ContIn,ContOut,AssocInter,AssocOut,Link).


chercher_le_bon(0,Pos,Dliste,A,ContIn,ContOut,Assoc,AssocOut,T<==>I):-
    ieme(Pos,Dliste,I),
    identifier(A,ContIn,ContOut,Assoc,AssocOut,T<==>I).
chercher_le_bon(Val,Pos,Dliste,A,C1,C5,Assoc,AssocOut,Link):-
    Val < 0,
    P is Pos-1,
    ieme(P,Dliste,I),
    update_cont(T<==>I,C1,C2,Assoc,A2),
    (identifier(A,C2,C3,Assoc,_,T<==>I) ->
        NewVal is Val + 1, C4 = C3;
        NewVal is Val, C4 = C2),
    chercher_le_bon(NewVal,P,Dliste,A,C4,C5,A2,AssocOut,Link).
chercher_le_bon(Val,Pos,Dliste,A,C1,C5,Assoc,AssocOut,Link):-
    Val > 0,
    P is Pos+1,
    ieme(P,Dliste,I),
    update_cont(T<==>I,C1,C2,Assoc,A2),
    (identifier(A,C2,C3,Assoc,_,T<==>I) ->
         NewVal is Val - 1, C4 = C3;
         NewVal is Val, C4 = C2),
    chercher_le_bon(NewVal,P,Dliste,A,C4,C5,A2,AssocOut,Link).   


% L'instant est une sommation d'instants discontinus, ces derniers
% etant obtenu en appliquant un filtre sur un instant B
% On identifie un cadre <B,U,Dlist> et on extrait de cette 
% D-list tous les temps pouvant etre denommes par N
identifier(filtre(U,M,B),ContIn,[T<==>somme(I)|ContInter],
           AssocIn,[T<==>somme(I)|AssocInter],T<==>somme(I)):-
    identifier(B,ContIn,ContInter,AssocIn,AssocInter,(_<==>I2)),     
    def_cadre(_,cadre(I2,U,Dlist)),
    app_filtre(M,Dlist,ContInter,AssocInter,NouvListe,T),
    def_cadre(1,cadre(somme(I),U,NouvListe)).




%app_filtre([T,U,N],Dliste,Cont,Assoc,NouvListe,T):-
%    nonvar(N),
%    denom_multivoque(N),!,
%    app_filtre1([T,U,N],Dliste,Cont,Assoc,NouvListe,T).
%app_filtre([T,U,N],Dliste,Cont,Assoc,NouvListe,T):-
%    app_filtre2([T,U,N],Dliste,Cont,Assoc,NouvListe,T).


app_filtre(_,[],_,_,[],_).
% Cas ou on a affaire a une denommination simple
app_filtre(M,Dliste,Cont,Assoc,NouvListe,T):-
    M = [T,U,N],
    ieme(1,Dliste,Premier),
    (noms_possibles2(Premier,[],Assoc,_,Cont,Noms),
          member(incldans([_,U,N,_],_),Noms),
          NouvListe=[Premier|Liste];
    %sinon
          NouvListe=Liste),
    reste(Dliste,DlisteReste),
    app_filtre(M,DlisteReste,_,_,Liste,T),!.
    
% Cas ou on a affaire a une denomination complexe
app_filtre(M,Dliste,Cont,Assoc,NouvListe,T):-
    M \== [T,U,N],
    ieme(1,Dliste,Premier),
    (identifier(M,Cont,_,Assoc,_,T<==>Premier),
          NouvListe=[Premier|Liste];
     %sinon
          NouvListe=Liste),
     reste(Dliste,DlisteReste),
     app_filtre(M,DlisteReste,Cont,Assoc,Liste,T),!.


identifier(souscadre([T,U,_],Deb,Fin,B),ContIn,[T<==>somme(S2)|ContInter],
           AssocIn,[T<==>somme(S2)|AssocInter],T<==>somme(S2)):-
     identifier(Deb,ContIn,_,AssocIn,_,_<==>Ideb),
     identifier(Fin,ContIn,_,AssocIn,_,_<==>Ifin),
     identifier(B,ContIn,ContInter,AssocIn,AssocInter,_<==>Iref),
     def_cadre(_,cadre(Iref,U,DlisteRef)),
     premier(DlisteRef,Prem),
     dernier(DlisteRef,Der),
     (Ideb \== Prem -> precedent(Ideb,DlisteRef,PrecIdeb),
                       sousliste(Prem,PrecIdeb,DlisteRef,SubDlist1),
                       def_cadre(_,cadre(somme(S1),U,SubDlist1));
                       SubDlist1 = []),
     (Ifin \== Der  -> suivant(Ifin,DlisteRef,SuivIfin),
                       sousliste(SuivIfin,Der,DlisteRef,SubDlist3),
                       def_cadre(_,cadre(somme(S3),U,SubDlist3));
                       SubDlist3 = []),
     sousliste(Ideb,Ifin,DlisteRef,SubDlist2),
     def_cadre(_,cadre(somme(S2),U,SubDlist2)),
     def_cadre(_,cadre(Iref,partage(U,_),[somme(S1),somme(S2),somme(S3)])).

     




% Chercher un cadre <Ref,U,L> et une fonction F associee a ce cadre.
% Cette fonction est applique en analyse au nom N afin d'identifier 
% l'instant Inst qui lui correspond
identifier2(Ref,U,N,Inst):-
    def_cadre(_,cadre(Ref,U,L)),
    fass(cadre(Ref,U,L),F),
    domaine_compatible(F,N),
    appf(ana,F,Inst,cadre(Ref,U,L),N,_).

domaine_compatible(_,N):-
    var(N),!.
domaine_compatible(F,N):-
    domaine(F,Dom),
    element(N,Dom),!.
%domaine_compatible(F,N):-
%    in_domaine(F,N).


% On verifie qu'il existe un cadre tel que Inst peut etre denomme N
% dans ce cadre, et que Inst est inclus dans Ref
identifier3(Ref,_,N,Inst):-
    cadres_imm_englobants(Inst,LesCadres,[]),
    member(C,LesCadres),
    fass(C,F),
    domaine_compatible(F,N),
    appf(gen,F,Inst,C,N,_),
    inclus(Inst,Ref).

    

