

%%% Programme permettant la conversion d'une date          %%%
%%% en nombre de jours ecoules depuis le debut de l'an 0.  %%%
%%%                                                        %%%
%%% Julie Vachon (aout 1992)                               %%%
%%%
%%% Modifie et debugge par Michel Gagnon (Fevrier 1994)    %%%


% Exemple:
%
% date_a_jours(13/4/1971,NbJours).
% NbJours = 719999 
%
% jours_a_date(719999,J/M/A).
% J = 13,
% M = 4,
% A = 1971 

 
ecart(Date1,Date2,Ecart):-
    date_a_jours(Date1,J1),
    date_a_jours(Date2,J2),
    Ecart is J2-J1.


est_bissextile(Annee):-
    0 is Annee mod 100,!,
    0 is Annee mod 400.

est_bissextile(Annee):-
    0 is Annee mod 4.

mois(12,334).
mois(11,304).
mois(10,273).
mois(9,243).
mois(8,212).
mois(7,181).
mois(6,151).
mois(5,120).
mois(4,90).
mois(3,59).
mois(2,31).
mois(1,0).

moisB(N,J):-
    mois(N,JJ),
    \+ N is 1,
    \+ N is 2,
    J is JJ+1.
moisB(2,31).
moisB(1,0).

cycle(400,146097).
cycle(100,36524).
cycle(4,1461).
cycle(1,365).


cycle_inferieur(400,100).
cycle_inferieur(100,4).
cycle_inferieur(4,1).
cycle_inferieur(1,0).



mois_et_jour(A,Reste,M,J):-
    est_bissextile(A),!,
    moisB(M,NbJours), 
    Reste > NbJours,
    J is Reste-NbJours,!.
      
mois_et_jour(_,Reste,M,J):-
    mois(M,NbJours), 
    Reste > NbJours,
    J is Reste-NbJours,!.



%***********************************
%***********************************

date_a_jours(J/M/0,Jours):-
    moisB(M,XjMois),
    Jours is XjMois+J.
date_a_jours(J/M/A,Jours):-
    A > 0,
    cycle(400,Xjours),
    X1 is (A-1) // 400,
    X2 is (A-1) mod 400,
    X2nonbis is X2 // 100,
    X3bis is X2 // 4 - X2nonbis,
    X4nonbis is X2-X3bis,
    (est_bissextile(A) -> moisB(M,XjMois); mois(M,XjMois)),
    Jours is X1*Xjours + X3bis*366 + X4nonbis*365 + XjMois + J.



mois_a_jours(X,Jours):-
    Annee is X // 12,
    LeMois is X - Annee* 12,
    (LeMois = 0, M = 12, A=Annee; M = LeMois, A is Annee+1),
    date_a_jours(1/M/A,Jours).




%jours_a_date(J,Q/M/0):-
%    J =< 366,
%    mois_et_jour(0,J,M,Q).
jours_a_date(J,D):- 
    jours_a_date(400,J,D,1).

    

jours_a_date(0,Jours,J/M/A,A):-
    !,mois_et_jour(A,Jours,M,J).
jours_a_date(1,Jours,J/M/A,Acc):-
    !,A1 is Jours // 365,
    A1R is Jours mod 365,
    ( A1R = 0 ->  A is Acc + A1 -1 ,
                  M = 12,
                  (est_bissextile(A) -> J=30; J=31);
                  Acc2 is Acc+ A1,
                  jours_a_date(0,A1R,J/M/A,Acc2)).
jours_a_date(Cycle,Jours,J/M/A,Acc):-
    cycle(Cycle,C),
    A1 is Jours // C,
    A1R is Jours mod C,
    ( A1R = 0 ->  J=31,M=12,A is Acc + A1*Cycle - 1;
                  Acc2 is Acc+Cycle*A1,
                  cycle_inferieur(Cycle,Cinf),
                  jours_a_date(Cinf,A1R,J/M/A,Acc2)).



jours_a_mois(Jours,Mois):-
    jours_a_date(Jours,_/M/A),
    Mois is (A-1)*12+M.

% Pour determiner le jour de la semaine

id_jour_semaine(Date,Jour):-
    date_a_jours(Date,J),
    Ecart is J - 367,
    E is Ecart mod 7,
    associer_jour(E,Jour).
    


associer_jour(3,lundi).
associer_jour(4,mardi).
associer_jour(5,mercredi).
associer_jour(6,jeudi).
associer_jour(0,vendredi).
associer_jour(1,samedi).
associer_jour(2,dimanche).



% Pour determiner l'ecart en jours:

ecart(D1,D2,Ecart):-
    date_a_jours(D1,J1),
    date_a_jours(D2,J2),
    Ecart is J2 - J1.

    
