

% FONCTIONS DE DESIGNATION

% predicat appf(Orient,Fonct,Temps,Cadre,Nom,Ref)
% Orient precise si la fonction Fonct est appliquee en analyse 
% ou en generation
% Ref est un temps de reference qui peut etre utilise pour nommer un
% temps




% FONCTION f0
% Elle retourne la position du temps I dans la D-liste du cadre
% en fonction de l'ordre numerique de gauche a droite et en 
% comptant a partir de 1.

% En generation
appf(gen,f0,I,cadre(_,_,L),Nom,_):-
    nonvar(I),
    % Si Nom est liee, ca doit etre un entier
    (nonvar(Nom)-> integer(Nom); true),
    ieme(Nom,L,I).
% En analyse
appf(ana,f0,I,cadre(_,_,L),Nom,_):-
    nonvar(Nom),
    Nom > 0,
    ieme(Nom,L,I).

domaine(f0,entiers).



% FONCTION f0a
% Comme f0 mais compte a partir de zero

appf(gen,f0a,I,C,Nom,_):-
    appf(gen,f0,I,C,N,_),
    Nom is N-1.

appf(ana,f0a,I,C,Nom,_):-
    N is Nom+1,
    appf(ana,f0,I,C,N,_).

domaine(f0a,entiers).



% FONCTION f0b
% Comme f0 mais compte de droite a gauche.
% Le nom est un nombre negatif
% ex: -1 est le dernier, -2 est l'avant-dernier, etc.

% En generation
appf(gen,f0b,I,cadre(_,_,L),Nom,_):-
    longueur(L,Long),
    ieme(Pos,L,I),
    Nom is Pos-Long-1.

% En analyse
appf(ana,f0b,I,cadre(_,_,L),Nom,_):-
    longueur(L,Long),
    Pos is Long+Nom+1,
    ieme(Pos,L,I).


domaine(f0b,entiersnegatifs).


% FONCTION f1
% Nomme les jours d'une semaine
    
% En generation
appf(gen,f1,J,cadre(_,_,L),Nom,_):-
    ieme(I,L,J),
    I1 is I-1,
    liste_lex(f1,I1,Nom).

% En analyse
appf(ana,f1,J,cadre(_,_,L),Nom,_):-
    liste_lex(f1,I,Nom),
    I1 is I+1,
    ieme(I1,L,J).


liste_lex(f1,0,lundi).
liste_lex(f1,1,mardi).
liste_lex(f1,2,mercredi).
liste_lex(f1,3,jeudi).
liste_lex(f1,4,vendredi).
liste_lex(f1,5,samedi).
liste_lex(f1,6,dimanche).


domaine(f1,[lundi,mardi,mercredi,jeudi,vendredi,samedi,dimanche]).


% FONCTION f2
% Pour nommer les saisons
% Comme la fonction n'est pas univoque, il n'y a pas de definition
% en analyse

% En generation
appf(gen,f2,saison1(X),_,S,_):-
    nonvar(X),
    X =< 4,
    X1 is X-1,
    liste_lex(f2,X1,S).
appf(gen,f2,saison1(X),_,S,_):-
    nonvar(X),
    X > 4,
    X1 is (X-1) mod 4 + 1,
    appf(gen,f2,saison1(X1),_,S,_).

liste_lex(f2,0,hiver).
liste_lex(f2,1,printemps).
liste_lex(f2,2,ete).
liste_lex(f2,3,automne).


domaine(f2,[hiver,printemps,ete,automne]).



% FONCTION f3
% Designe chaque jour de l'annee par une fete

% En generation
appf(gen,f3,jour1(J),_,N,_):-
    var(N),
    jours_a_date(J,Q/M/_),
    liste_lex(f3,Q,M,N).
% En analyse
appf(ana,f3,jour1(J),cadre(annee1(A),_,_),N,_):-
    \+ var(N),
    liste_lex(f3,Q,M,N),
    date_a_jours(Q/M/A,J).

liste_lex(f3,1,1,jour_de_l_an).
liste_lex(f3,25,12,noel).
liste_lex(f3,_,_,saint_machin).

domaine(f3,[noel,jour_de_l_an]).

% FONCTION f4
% Designe les mois

% En generation
appf(gen,f4,mois1(M),cadre(_,_,mois1(D,_)),Nom,_):-
    M1 is M-D,
    liste_lex(f4,M1,Nom).
% En analyse
appf(ana,f4,mois1(M),cadre(_,_,mois1(D,_)),Nom,_):-
    liste_lex(f4,M1,Nom),
    M is M1+D.

    
liste_lex(f4,0,janvier).
liste_lex(f4,1,fevrier).
liste_lex(f4,2,mars).
liste_lex(f4,3,avril).
liste_lex(f4,4,mai).
liste_lex(f4,5,juin).
liste_lex(f4,6,juillet).
liste_lex(f4,7,aout).
liste_lex(f4,8,septembre).
liste_lex(f4,9,octobre).
liste_lex(f4,10,novembre).
liste_lex(f4,11,decembre).


domaine(f4,[janvier,fevrier,mars,avril,mai,juin,juillet,aout,septembre,
            octobre,novembre,decembre]).


% FONCTION f5
% Designe les jours et les nuits
% Comme la fonction n'est pas univoque, il n'y a pas de definition
% en analyse

% En generation
appf(gen,f5,journuit1(X),_,N,_):-
    (N=jour, pair(X));
    (N=nuit, impair(X)).

domaine(f5,[nuit,jour]).


% FONCTION f6
% Designe les moments de la journee

% En generation
appf(gen,f6,M,cadre(_,_,L),N,_):-
    ieme(I,L,M),
    liste_lex(f6,I,N).
% En analyse
appf(ana,f6,M,cadre(_,_,L),N,_):-
    liste_lex(f6,I,N),
    ieme(I,L,M).

liste_lex(f6,1,nuit).
liste_lex(f6,2,matin).
liste_lex(f6,3,apresmidi).
liste_lex(f6,4,soir).

domaine(f6,[nuit,matin,apresmidi,soir]).


% FONCTION f7
% Pour les jours en relation avec le moment d'enonciation
%
% En generation
%appf(gen,f7,J,cadre(ere1(1),jour,Dlist),N,_):-
%    enonciation(E),
%    projection(E,cadre(ere1(1),jour,Dlist),E2,[]),
%    position(E2,PosE2),
%    position(J,PosJ),
%    PosJ2 is PosJ-PosE2,
%    liste_lex(f7,PosJ2,N).
%
% En analyse
%appf(ana,f7,J,cadre(ere1(1),jour,Dlist),N,_):-
%    enonciation(E),
%    projection(E,cadre(ere1(1),jour,Dlist),E2,[]),
%    position(E2,PosE2),
%    liste_lex(f7,Pos,N),
%    PosJ is Pos+PosE2,
%    ieme(PosJ,Dlist,J).
%
%liste_lex(f7,-2,avanthier).
%liste_lex(f7,-1,hier).
%liste_lex(f7,0,aujourdhui).
%liste_lex(f7,1,demain).
%liste_lex(f7,2,apresdemain).
%liste_lex(f7,N,N):- 
%     \+ member(N,[-2,-1,0,1,2]).
%
%
%domaine(f7,[avanthier,hier,aujourdhui,demain,apresdemain]).


% FONCTION f8
% Designe le temps par rapport a un temps de reference
% Represente des formes telles que
%     "il y a n Ntemps", "n Ntemps apres" (dans le sens de "le nieme
%     Ntemps  avant/apres celui de reference)
%
%     " le Ntemps precedent", "Ntemps prochain"    

% En generation
% Il y a plusieurs temps de reference possibles
% On suppose donc ici que la valeur de Ref est donnee.
%appf(gen,f8,Temps,cadre(_,U,Dlist),[_,U,PosRel],Ref):-
%    nonvar(Ref),
%    ieme(Pos,Dlist,Temps),
%    ieme(PosRef,Dlist,Ref),
%    PosRel is Pos - PosRef.
%
% En analyse
%appf(ana,f8,Temps,cadre(_,U,Dlist),[_,U,PosRel],Ref):-
%    num_rel(Ref,Dlist,D2),
%    chercher_posrel(PosRel,D2,Temps).


% Cas de liste standard
chercher_posrel(Pos,L,I):-
    member((I,Pos),L).
% Cas de liste abregee
chercher_posrel(0,(_,R,_),R):- 
    R \== nil.
chercher_posrel(N,(G,_,_),I):- 
    N<0,
    G\==[],
    const_labregee(Nom,_,F,G),
    P is F+N+1,
    decomp_elem(I,Nom,P),
    element(I,G).
chercher_posrel(N,(_,_,D),I):- 
    N>0,
    D\==[],
    const_labregee(Nom,X,_,D),
    P is X+N-1,
    decomp_elem(I,Nom,P),
    element(I,D).    




%in_domaine(f8,[X,U,I,_]):-
%    var(X),
%    member(U,[jour,semaine,mois,saison,annee]),
%    element(I,entiers).
%   

% FONCTION f9


% En generation


% En analyse
%appf(ana,f9,J,cadre(ere1(1),U,Dlist),[N,U,PosRel,Ref]):-
%    ieme(PosRef,Dlist,Ref),
%    (U=jour,  Card=7;
%     U=mois,  Card=12;
%     U=saison,Card=4),
%    (PosRel>0, Pdeb is PosRef+(PosRel-1)*Card+1,
%               Pfin is Pdeb+Card-1;
%     PosRel<0, Pfin is PosRef+(PosRel+1)*Card-1,
%               Pdeb is Pfin-Card+1),
%     chercher_le_bon(U,Dlist,Pdeb,Pfin,N,J).
%
%chercher_le_bon(U,Dlist,Pdeb,Pfin,N,Tcherche):-
%     Pdeb =< Pfin,
%     ieme(Pdeb,Dlist,Tcherche),
%     (U=jour, CadRef=semaine1(X);
%      U=mois, CadRef=annee1(X);
%      U=saison,CadRef=ere1(X)),
%     cadre_sup(Tcherche,cadre(CadRef,U,L),[]),
%     fass(cadre(CadRef,U,L),F),
%     appf(gen,F,Tcherche,cadre(CadRef,U,L),N),!.
%chercher_le_bon(U,Dlist,Pdeb,Pfin,N,X):-
%     Pdeb < Pfin,
%     P is Pdeb+1,
%     chercher_le_bon(U,Dlist,P,Pfin,N,X).
%     
%
%in_domaine(f9,[N,_,P,_]):-
%     nonvar(N),
%     integer(P),
%     (domaine(f1,D),member(N,D);
%      domaine(f4,D),member(N,D)).
%
%
%



% DIVERS




% Determine si un nom correspond a une fonction de designation
% multivoque
denom_multivoque(N):-
    domaine(f1,Dom), member(N,Dom);
    domaine(f2,Dom), member(N,Dom).
