
% IDENTIFICATION DU TYPE DE LISTE

listestandard([]).
listestandard([_|_]).

listeabregee(L):-
   \+ listestandard(L),
   L =.. [Nom,_,_],
   Nom \== '.'.



% IDENTIFICATION DE LA POSITION D'UN ELEMENT DANS UNE LISTE ABREGEE

position(X,P):-
   arg(1,X,P).

% IDENTIFICATION DU NOM DE LISTE ABREGEE

nomliste(L,Nom):-
    L =.. [Nom|_].


% DECOMPOSITION D'UN ELEMENT DE LISTE ABREGEE
decomp_elem(E,Nom,Pos):-
    E =.. [Nom,Pos].




% CONSTRUCTION D'UNE LISTE SOUS FORME ABREGEE

% A partir du nom et des numeros de debut et de fin
const_labregee(Nom,D,F,Labr):-
   Labr =.. [Nom,D,F].


% A partir de deux elements d'une meme liste
fill_labregee(D,F,Labr):-
   D =.. [Nom,X],
   F =.. [Nom,Y],
   Labr =.. [Nom,X,Y].




% EXPANSION DE LISTES
% Fait l'expansion de la forme abregee en forme standard

% Si deja sous forme standard on ne fait rien
expand([],[]).
expand([X|Y],[X|Y]).
% Si la liste est ecrite sous la forme nom(Debut,Fin)
expand(L1,L2):-
    L1 =.. [Nom,D,F],
    expand2(Nom,D,F,L2).

expand2(Nom,X,X,[L2]):-
    L2 =.. [Nom,X].
expand2(Nom,D,F,[Elem|L]):-
    D \== F,
    Elem =.. [Nom,D],
    D1 is D+1,
    expand2(Nom,D1,F,L).



% RETOURNE LA TAILLE D'UNE LISTE

longueur(X,N):-
    listestandard(X),
    length(X,N).
longueur(X,N):-
    listeabregee(X),
    X =.. [_,D,F],
    N is F-D+1.

% RETOURNE LE IEME ELEMENT D'UNE LISTE

% Sous forme abregee
% On cherche X
ieme(I,L,X):-
    nonvar(I),
    listeabregee(L),
    L =.. [Nom,D,F],
    I2 is D+I-1,
    I2 >= D,
    I2 =< F,
    X =.. [Nom,I2].
ieme(I,L,X):-
    nonvar(I),
    \+ listeabregee(L),
    nth(I,L,X).

% On cherche I
ieme(I,L,X):-
    nonvar(X),
    listeabregee(L),
    X =.. [Nom,P],
    L =.. [Nom,D,F],
    P >= D,
    P =< F,
    I is P-D+1.
ieme(I,L,X):-
    nonvar(X),
    \+ listeabregee(L),
    nth(I,L,X).


    

% RECHERCHE l'ELEMENT MEDIAN D'UNE LISTE

% Sous forme abregee
milieu(L,M):-
    L =.. [Nom,D,F],
    M1 is (D+F) // 2,
    M =.. [Nom,M1].


% CONCATENATION DE LISTE
% Si deja sous forme standard
mon_concat_liste(X,Y,Z):-
    listestandard(X),
    listestandard(Y),
    append(X,Y,Z),!.
% Si sous forme abregee
mon_concat_liste(L,[],L).
mon_concat_liste([],L,L).
mon_concat_liste(L1,L2,L3):-
    L1 =.. [Nom,D1,F1],
    L2 =.. [Nom,D2,F2],
    D2 is F1+1,
    L3 =.. [Nom,D1,F2],!.

mon_concat_liste(L1,L2,L3):-
    expand(L1,L1exp),
    expand(L2,L2exp),
    append(L1exp,L2exp,L3).


% FUSION D'UN ENSEMBLE DE LISTES
mon_merge([],[]).
mon_merge([X|Xs],R):-
    mon_merge(Xs,Xm),
    mon_concat_liste(X,Xm,R).



% Elimine tous les elements d'une liste qui se retrouve
% deja dans une autre liste donnee

eliminer_existants([],_,[]).
eliminer_existants([C|Cs],L,Cz):-
    member(C,L),
    eliminer_existants(Cs,L,Cz).
eliminer_existants([C|Cs],L,[C|Cz]):-
    eliminer_existants(Cs,L,Cz).



% ELEMENT D'UNE LISTE
% cas speciaux
element(X,entiers):-
    !,integer(X).
element(X,entiersnegatifs):-
    !,integer(X),
    X<0.
% Si sous forme standard
element(X,L):-
    member(X,L).
% Si sous forme abregee
element(X,L):-
    listeabregee(L),
    X =.. [Nom,No],
    nonvar(No),
    L =.. [Nom,D,F],
    No >= D,
    No =< F.
element(X,L):-
    listeabregee(L),
    X =.. [Nom,No],
    var(No),
    (L =.. [Nom,No,_]; reste(L,L2), element(X,L2)).

% PREMIER (DERNIER) ELEMENT D'UNE LISTE

% Si sous forme standard
premier([X|_],X).
dernier([X],X).
dernier([_|Xs],Last):-dernier(Xs,Last).

% Si sous forme abregee
premier(L,First):-
    \+ listestandard(L),
    L =.. [Nom,Deb,_],
    First =.. [Nom,Deb].
dernier(L,Last):-
    \+ listestandard(L),
    L =.. [Nom,_,Fin],
    Last =.. [Nom,Fin].



% PRECEDENT ET SUIVANT

% Liste abregee
precedent(X,L,Prec):-
     listeabregee(L),
     ieme(Pos,L,X),
     P2 is Pos-1,
     ieme(P2,L,Prec).

suivant(X,L,Suiv):-
     listeabregee(L),
     ieme(Pos,L,X),
     P2 is Pos+1,
     ieme(P2,L,Suiv).
     



% RESTE D'UNE LISTE

% si sous forme standard
reste([],[]):-!.
reste([_|X],X):-!.
% Si sous forme abregee
reste(L,[]):-
    \+ listestandard(L),
    L =.. [_,X,X],!.
reste(L,Reste):-
    \+ listestandard(L),
    L =.. [Nom,Deb,Fin],
    Deb\==Fin,
    NouvDeb is Deb+1,
    Reste =.. [Nom,NouvDeb,Fin].



% Intersection de deux listes

intersection(L1,L2,Inter):-
    expand(L1,X),
    expand(L2,Y),
    intersec(X,Y,Inter).
intersec([],_,[]).
intersec([X|Xs],Y,[X|Zs]):-
    member(X,Y),!,
    intersec(Xs,Y,Zs).
intersec([_|Xs],Y,Zs):-
    intersec(Xs,Y,Zs).



% SOUSLISTE
% La liste L1 est une sous-liste de L2

sousliste(X,Y):-
    listestandard(X),
    sousliste2(X,Y).

sousliste(X,Y):-
    listeabregee(X),
    sousliste3(X,Y).

sousliste2([],_).
sousliste2(L1,L2):-
    prefix(L1,L2).
sousliste2(L1,[_|L2]):-
    sousliste2(L1,L2).

sousliste3(X,Y):-
    X =..[N,Dx,Fx],
    Y =..[N,Dy,Fy],
    Dy =< Dx,
    Fx =< Fy.


% IDENTIFICATION DE SOUS-LISTE
% On donne les elements de debut de de fin de la sous-liste

sousliste(D,F,L,SousL):-
    listestandard(L),!,
    slstd(D,F,L,SousL).
sousliste(D,F,L,SousL):-
    listeabregee(L),
    slabr(D,F,L,SousL).

slstd(D,F,[D|Ls],[D|SousL]):-
    !,inserer_jusqua(F,Ls,SousL).
slstd(D,F,[_|Ls],SousL):-
    slstd(D,F,Ls,SousL).

inserer_jusqua(F,[F|_],[F]):-!.
inserer_jusqua(F,[L|Ls],[L|SousL]):-
    inserer_jusqua(F,Ls,SousL).


slabr(D,F,L,SousL):-
    D=..[Nom,Deb],
    F=..[Nom,Fin],
    Deb =< Fin,
    SousL =..[Nom,Deb,Fin],
    sousliste(SousL,L).

    

% NUMEROTATION DE LISTE
% Numerote les element d'une liste par rapport a un referent
% negatif vers la gauche positif vers la droite par rapport
% a ce referent
% [a,b,c,d,e] d --> [(a,-3),(b,-2),(c,-1),(d,0),(e,1)]
% Le referent n'est pas necessairement contenu dans la liste
% Une relation d'ordre est requise

num_rel(Ref,Lin,Lout):-
    listestandard(Lin),
    num_rel1(Lin,Ref,_,Lout),!.
num_rel(Ref,Lin,Lout):-
    listeabregee(Lin),
    num_rel2(Ref,Lin,Lout),!.

num_rel1([],_,-1,[]).
num_rel1([X|Xs],Ref,Nout,[(X,N)|Ys]):-
    avant(X,Ref),
    num_rel1(Xs,Ref,N,Ys),
    Nout is N-1.
num_rel1([X|Xs],X,-1,[(X,0)|Ys]):-
    num1(Xs,1,Ys).
num_rel1(X,_,-1,Y):-
    num1(X,1,Y).

num1([],_,[]).
num1([X|Xs],N,[(X,N)|Ys]):-
    N2 is N+1,
    num1(Xs,N2,Ys).

num_rel2(Ref,Lin,(Lin,nil,[])):-
    Ref=..[_,X],
    Lin=..[_,_,Y],
    Y < X.
num_rel2(Ref,Lin,([],nil,Lin)):-
    Ref=..[_,X],
    Lin=..[_,Y,_],
    X < Y.
num_rel2(Ref,Lin,(G,Ref,D)):-
    Ref =..[_,R],
    Lin =..[N,X,Y], 
    X1 is R-1,
    Y1 is R+1,
    (X1>=X -> G =..[N,X,X1]; G=[]),
    (Y1=<Y -> D =..[N,Y1,Y]; D=[]).  

    




% ELIMINATION DE REDONDANCE
% Elimine dans une liste toutes les occurences multiples d'un 
% meme element

elim_redond([],[]).
elim_redond([X|Xs],Xs2):-
    member(X,Xs),
    elim_redond(Xs,Xs2).
elim_redond([X|Xs],[X|Xs2]):-
    elim_redond(Xs,Xs2).

% IDENTIFICATION DU TYPE DE LA LISTE

type(L,Type):-
    functor(L,Nom,_),
    liste(Nom,_,Type),!.


% VERIFICATION DE TYPE
% Deux listes (ou un element et une liste) sont du meme type
% L'ecriture doit etre sous la forme abregee
meme_type([X|_],[Y|_]):-
    meme_type(X,Y).
meme_type(X,[Y|_]):-
    meme_type(X,Y).
meme_type([X|_],Y):-
    meme_type(X,Y).
meme_type(L1,L2):-
    type(L1,T),
    type(L2,T).



enlever_dupl([],[]).
enlever_dupl([X|Xs],[X|Z]):-
    \+ member(X,Xs),!,
    enlever_dupl(Xs,Z).
enlever_dupl([_|Xs],Z):-
    enlever_dupl(Xs,Z).

