% IMPRIMEUR
% Compose la forme finale du texte apres lui avoir fait subir des 
% ajustements syntaxiques

% Ce fichier contient les clauses suivantes:
%                       ecrire
%                       former_texte
%                       imprimer
%                       filtrer
%                       majuscule
%                       commence_voy
%                       special



ecrire(Texte):-
   filtrer(Texte,T1),
   elisions(T1,T2),
   contractions(T2,T3),
   ajust_morpho(T3,T4),
   T4=[Prem|Reste],
   majuscule(Prem,P2),
   former_texte(0,[P2|Reste]),!.

filtrer([],[]).
% Eliminer les valeurs vides qui representent les ellipses
filtrer([#X|R],R2):-
    const(X,_,S,_),
    extraire_trait(S,lex,[]),
    filtrer(R,R2).
filtrer([#X|R],[ValLex|R2]):-
    const(X,_,S,_),
    extraire_trait(S,lex,ValLex),
    filtrer(R,R2).



majuscule(Mot,Motmaj):-
   \+ special(Mot),
   name(Mot,[Prem|Autres]),
   Prem >= 97,
   Prem =< 122,
   Premaj is Prem - 32,
   name(Motmaj,[Premaj|Autres]).
majuscule(Mot,Motmaj):-
   \+ special(Mot),
   name(Mot,[Lettre|Autres]),
   name(L,[Lettre]),
   conv_maj(L,Lmaj),
   name(Lmaj,[LettreMaj]),
   name(Motmaj,[LettreMaj|Autres]).
majuscule(Mot,Mot).



elisions([],[]).
elisions([Particule,Mot|R1],[Z|R2]):-
	member(Particule,[de,le,la,se,'Jusque',jusque]),
	name(Mot,[Prem|Autres]),
        name(X,[Prem]),
	voyelle(X),
	name(Particule,Pn),
        retirer_dernier(Pn,Pn2),
        merge([Pn2,[39],[Prem|Autres]],NouvMot),
	name(Z,NouvMot),
	elisions(R1,R2).
elisions([X|R1],[X|R2]):-
	elisions(R1,R2).
	
	
	
contractions([],[]).
contractions([X],[X]).
contractions([X,Y|Reste],[Z|Reste2]):-
	contracter(X,Y,Z),
        contractions(Reste,Reste2).
contractions([X|Reste],[X|Reste2]):-
        contractions(Reste,Reste2).

contracter('',le,au).
contracter('',les,aux).
contracter(de,le,du).
contracter(de,les,des).



ajust_morpho([],[]).
ajust_morpho([X,Y|Reste],[X2,Y|Reste2]):-
	member(X,[ce,beau]),
        commence_voy(Y),
        modif_morpho(X,X2),
        ajust_morpho(Reste,Reste2).
ajust_morpho([X|Reste],[X|Reste2]):-
        ajust_morpho(Reste,Reste2).


modif_morpho(ce,cet).
modif_morpho(beau,bel).

former_texte(_,[]).

former_texte(_,[afficher(Terme)|Reste]):-
   nl,   bel_aff(Terme),  nl,
   former_texte(0,Reste).

former_texte(Compteur,[Fin,Ponct]):-
   member(Ponct,['.','?','!']),
   nom(Fin,Nfin),
   append(Nfin,[Ponct,' '],T),
   nom(Txt,T),
   imprimer(Compteur,_,Txt).

former_texte(_,[parag]):-
   nl.

former_texte(Compteur,[Mot,Ponct|Reste]):-
   member(Ponct,[',',';']),
   nom(Mot,Nmot),
   append(Nmot,[Ponct,' '],T),
   nom(Txt,T),
   imprimer(Compteur,NouvComp,Txt),
   former_texte(NouvComp,Reste).

former_texte(_,[parag,Mot|Reste]):-
   majuscule(Mot,MotMaj),
   nl,nl,
   imprimer(0,NouvComp,'        '),
   former_texte(NouvComp,[MotMaj|Reste]).

former_texte(Compteur,[Fin,Ponct,Debut|Reste]):-
   member(Ponct,['.','?','!']),
   nom(Fin,Nfin),
   majuscule(Debut,Debutmaj),
   append(Nfin,[Ponct,' '],T),
   nom(Txt,T),
   imprimer(Compteur,NouvComp,Txt),
   former_texte(NouvComp,[Debutmaj|Reste]).

former_texte(Compteur,[Mot|Reste]):-
   nom(Mot,Nmot),
   append(Nmot,[' '],T),
   nom(Txt,T),
   imprimer(Compteur,NouvComp,Txt),
   former_texte(NouvComp,Reste).


imprimer(_,0,textuel(Texte)):-
   nl,
   write(Texte),
   nl.
imprimer(Compteur,NouvComp,Texte):-
   long_string(Texte,Long),
   Fin is Compteur + Long,
   Fin =< 75,
   write(Texte),
   NouvComp is Compteur+Long.
imprimer(_,Long,Texte):-
   nl,
   long_string(Texte,Long),
   write(Texte).
imprimer(_,0,Texte):-
   nl,
   write(Texte),
   nl.




commence_voy(Mot):-
   nom(Mot,[Lettre|_]),
   member(Lettre,[a,'','',e,'','','',i,'',o,'',u,'']).
commence_voy(Mot):-
   nom(Mot,[h|Reste]),   
   not h_aspire([h|Reste]).





h_aspire([h,,b,l,e|_]).
h_aspire([h,a,c,h|_]).
h_aspire([h,a,d,d,o,c,k|_]).
h_aspire([h,a,i,l,l,o,n|_]).
h_aspire([h,a,i,n|_]).
h_aspire([h,a,l,a,g,e|_]).
h_aspire([h,a,l,,t|_]).
h_aspire([h,a,l,l,e,b,a,r,d|_]).
h_aspire([h,a,l,o|_]).
h_aspire([h,a,l,t,e|_]).
h_aspire([h,a,m,e,a,u|_]).
h_aspire([h,a,m,p,e|_]).
h_aspire([h,a,n,c,h,e|_]).
h_aspire([h,a,n,d|_]).
h_aspire([h,a,n,g,a,r|_]).
h_aspire([h,a,n,n,e,t,o,n|_]).
h_aspire([h,a,n,t|_]).
h_aspire(X):-
	X=[h,a,r|_],
	X\==[h,a,r,m,o|_].
h_aspire([h,a,s|_]).
h_aspire([h,a,u|_]).
h_aspire([h,a,v|_]).
h_aspire([h,e,r,a,u|_]).
h_aspire([h,,r,i,s,s|_]).
h_aspire([h,,r,o,n|_]).
h_aspire([h,,r,o,s|_]).
h_aspire([h,,r,p,e|_]).
h_aspire([h,,r,s,e|_]).
h_aspire([h,,t,r,e|_]).
h_aspire([h,e,u,r,t|_]).
h_aspire([h,i,a,t,u,s|_]).
h_aspire([h,i,d,e,u|_]).
h_aspire([h,i,,r,a,r|_]).
h_aspire([h,i,t,t,i,t,e|_]).
h_aspire([h,o,c,h,e|_]).
h_aspire([h,o,c,k,e,y|_]).
h_aspire([h,o,m,a,r,d|_]).
h_aspire([h,o,n,g,r|_]).
h_aspire([h,o,q,u|_]).
h_aspire([h,o,r,s|_]).
h_aspire([h,o,u|_]).
h_aspire([h,u,e|_]).
h_aspire([h,u,|_]).
h_aspire([h,u,i,t,i|_]).
h_aspire([h,u,r,l|_]).







special(parag).
special(textuel(_)).
special(afficher(_)).



voyelle(X):- member(X,[a,,,e,,,,i,,o,,u,,'A','E','I','O','U','','','','']).

conv_maj(X,'A'):-member(X,[a,,]).
conv_maj(X,'E'):-member(X,[e,,,]).
conv_maj(X,'I'):-member(X,[i,]).
conv_maj(X,'O'):-member(X,[o,]).
conv_maj(X,'U'):-member(X,[u,]).

