:-dynamic mode_inspect/0.

composer([]):-  clean_all.
composer(Liste):-
        clean_all,
        init_contexte,
	compo(Liste).

compo([]).
compo([Even|Reste]):-
	generer(Even),!,
	compo(Reste).


generer(UniteSem):-
        creer_const([prop],UniteSem,[],IdProp),
        asserta(domine(racine,_,racine)),
	asserta(domine(racine,prop,IdProp)),
        extraire_trait(UniteSem,loc,L),
        calcule_cadre(L),
        nl,
	produire(IdProp,Resultat),!,
	ecrire(Resultat),nl.


produire(Id,Resultat):-
        const(Id,_,_,_),
        inspect(const,Id),
	assert(topic(Id)),
	asserta(domine(Id,constituant,Id)),
	debut(X),
        inspect(preselect,Id),
	traiter(debut(X)),!,
        inspect(selections,Id),
        structure(Id,Struct),
        inspect(structure,Id),
	app_lex(Id,Struct,StructLex),
        app_dec(StructLex),
	retract(topic(Id)),!,
	continuer(StructLex,Resultat).





% Creer un nouvel identificateur de constituant.
% Le premier argument est une liste de fonctions
% Le deuxieme argument est une liste de traits semantiques
% Le troisieme est une liste de traits preselectionnes
creer_const(F,S,T,Id):-
	gensym(Id),
        assert(const(Id,F,S,T)).

% Production de chaque item du pattern.

continuer([],[]).
continuer([#X|R1],[#X|R2]):- continuer(R1,R2).
continuer([Item|R],Resultat):-
	produire(Item,Prod),
	continuer(R,R2),
	append(Prod,R2,Resultat).





% Moteur pour l'interpretation d'une grammaire systemique

traiter(fin).

traiter(debut(X)):-
	tenter_realisation(debut,X),
	nouveau_trait(debut,X),
	suite(debut,X,Y),
	traiter(Y).
	
traiter(conj([])).
traiter(conj([X|Reste])):-
	traiter(X),
	traiter(conj(Reste)).

traiter(geler(Syst,C)):-
        connexion_complexe(C),
        evaluer_exp_selec(C),
	traiter(Syst).
traiter(geler(_,_)).



traiter(Nom):-
	topic(Id),
        selectionner_un_trait(Id,Nom,Trait),!,
	tenter_realisation(Nom,Trait),!,
	(suite(Nom,Trait,Suite),
	    traiter(Suite);

	    write('Je ne trouve pas la suite de  '),
	    write(Trait),nl,
	    write('dans le systeme  '),
	    write(Nom),nl).




tenter_realisation(Systeme,Trait):-
	realiser(Systeme,Trait).
tenter_realisation(Systeme,Trait):-
	write('erreur: Je suis incapable de realiser   '),
	write(Trait),
        write('   du systeme   '),
        write(Systeme),nl.



% Evalue une expression booleene sur un ensemble de traits qui doivent avoir
% ete selectionnes.


evaluer_exp_selec(T===V):-
	topic(Id),
	const(Id,_,_,Traits),
        member(T===V,Traits).
evaluer_exp_selec((X,Y)):-
	evaluer_exp_selec(X),
	evaluer_exp_selec(Y).
evaluer_exp_selec((X;Y)):-
	evaluer_exp_selec(X);
	evaluer_exp_selec(Y).
	




% Selection d'un trait. On verifie s'il n'est pas deje dans la 
% liste des traits. Sinon on declenche la regle de selection
% associee au systeme.       
selectionner_un_trait(Id,Nom,Choisi):-      
          const(Id,_,_,T),
          extraire_trait(T,Nom,Choisi).
selectionner_un_trait(Id,Nom,Trait):- 
          selection(Nom,Choisi),
	(Choisi=erreur -> 
           selection_usager(Id,Nom,Trait);
           Trait = Choisi),
          nouveau_trait(Nom,Trait).
  

% On demande a l'usager de choisir lui-meme un trait, parce
% que le systeme n'a pas reussi.
selection_usager(Id,Nom,T):-         
	write('Incapable de selectionner un trait dans le systeme '),
	write(Nom),nl,
        write('Constituant traite:  '),write(Id),nl,
        const(Id,_,Sem,_),write(Sem),nl,
        write('Choisissez: '),
        systeme(Nom,Liste,_),
        write(Liste),nl,
        read(T),
        member(T,Liste).



% Mise en attente d'un systeme jusqu'a ce que toutes les conditions d'entree
% soit respectees. 
creer_attente(Syst,N):-
	assert(attente(Syst,N)).
retirer_attente(Syst):-
	retract(attente(Syst,_)).
en_attente(Syst,N):-
	attente(Syst,N).
mettre_a_jour(Syst):-
	retract(attente(Syst,N)),
	N1 is N-1,
	assert(attente(Syst,N1)).


% Fonction de selection
% Applique, pour chaque trait de la liste, le test defini dans
% la grammaire. Le trait choisi est le premier dont le test reussit.
selection(S,Trait):-
	systeme(S,T,_),
	selection2(S,T,Trait).
selection(Systeme,Defaut):-
	systeme(Systeme,_,Defaut).

selection2(Systeme,[Prem|_],Prem):-
	test(Systeme,Prem).
selection2(Systeme,[_|Reste],Choisi):-
	selection2(Systeme,Reste,Choisi).


% Remplacement de toutes les fonctions d'un pattern par leurs valeurs
instancier(X,Y,Z):- instancier2(X,Y,Z).
instancier(_,L,_):-
	write('erreur: incapable d''instancier:'),nl,write(L),nl,fail.        


instancier2(_,[],[]).
instancier2(X,[F|Reste],[Val|ResteInst]):-
	domine(X,F,Val),
	instancier2(X,Reste,ResteInst).




% Unification de deux patterns.
% Le code "..." peut etre unifier avec n'importe quelle
% suite d'items.
unifier([],[],[]).
unifier(X,[...],X).
unifier([...],X,X):-
	X \== '...'.
unifier([...,X|R1],[...,X|R2],[...,X|R3]):-
	unifier(R1,R2,R3).
unifier([...,X|R1],[...,Y|R2],[...,X|R3]):-
	X \== Y,
	\+ member(X,R2),
	unifier(R1,[...,Y|R2],R3).
unifier([...,X|R1],[...,Y|R2],[...,Y|R3]):-
	X \== Y,
	\+ member(Y,R1),
	unifier([...,X|R1],R2,R3).
unifier([...,X|R1],[Y|R2],[Y|R3]):-
	Y \== ...,
	X \== Y,
	\+ member(Y,R1),
	unifier([...,X|R1],R2,R3).
unifier([X|R1],[...,Y|R2],[X|R3]):-
	X \== ...,
	X \== Y,
	\+ member(X,R2),
	unifier(R1,[...,Y|R2],R3).
unifier([...,X|R1],[X|R2],[X|R3]):-
	X \== ...,
	unifier(R1,R2,R3).
unifier([X|R1],[...,X|R2],[X|R3]):-
	X \== ...,
	unifier(R1,R2,R3).
unifier([X|R1],[X|R2],[X|R3]):-
	X \== ...,
	unifier(R1,R2,R3).



% Unifier un a un tous les patterns

structure(N,OrdreFinal):-
	setof(Z,pattern(N,Z),Liste),
	resoudre([...],Liste,Ordre),
	enlever_trous(Ordre,Ordre2),
	instancier(N,Ordre2,Ordre3),!,
        struct_decomp(Ordre3,OrdreFinal).
structure(N,[]):-
	write('erreur: Incapable d''identifier une structure'),nl,
        write('constituant:'),
        const(N,F,_,_),
	write(F),nl,
        write('contraintes:'),nl,
        setof(Z,pattern(N,Z),Liste),
        write(Liste),nl.

struct_decomp([],[]).
struct_decomp([X|Xs],[X|Ys]):-
        \+ pattern(X,_),
        struct_decomp(Xs,Ys).
struct_decomp([X|Xs],Result):-
        structure(X,StructX),
        struct_decomp(Xs,StructXs),
        append(StructX,StructXs,Result).

resoudre(P,[],P).
resoudre(P,[X|Reste],Q):-
	unifier(P,X,Y),
	resoudre(Y,Reste,Q).
resoudre(_,_,_):-
	topic(X),
        const(X,F,S,T),
	write('erreur: je suis incapable d''unifier les patterns pour  '),
	write(const(X,F,S,T)),nl,
	fail.

enlever_trous([],[]).
enlever_trous([...|R],R2):-enlever_trous(R,R2).
enlever_trous([X|R],[X|R2]):-enlever_trous(R,R2).


% Application de la procedure de lexicalisation
app_lex(_,[],[]).
app_lex(Id,[C|Reste],[#C|ResteLex]):-
        const(C,_,S,_),
        extraire_trait(S,lex,_),
        app_lex(Id,Reste,ResteLex).
app_lex(Id,[C|Reste],[#C|ResteLex]):-
        const(C,_,S,_),
        const(Id,_,_,Traits),
        extraire_trait(S,catlex,Cat),
        extraire_trait(S,message,Sem),
	lex(Cat,Sem,Traits,Val),
        retract(const(C,F,S,T)),
        assert(const(C,F,[lex===Val | S],T)),
	app_lex(Id,Reste,ResteLex).
app_lex(Id,[F|Reste],[F|ResteLex]):-
	app_lex(Id,Reste,ResteLex).



% Application de la procedure de declinaison
app_dec([]).
app_dec([#X | Reste]):-
        const(X,F,S,T),
        extraire_trait(S,catlex,Cat),
        extraire_trait(S,lex,Lex),
        extraire_trait(S,decliner,oui),
        decliner(Cat,Lex,NouvLex),
        remplace(lex===Lex,lex===NouvLex,S,S2),
        retract(const(X,F,S,T)),
        assert(const(X,F,S2,T)),
        app_dec(Reste).
app_dec([_| Reste]):- app_dec(Reste).

	

clean:-
	eliminer(choisi,3),
	eliminer(pattern,3),
	eliminer(domine,3),
	eliminer(topic,1),
	eliminer(attente,2),
	eliminer(pattern,2),
	eliminer(message,1).


%effacer_tableau(Symbole):-
%        effacer_les_fonctions(Symbole).
%
%effacer_les_fonctions(S):-
%	retract(fonction(S,_,_)),
%	fail.
%effacer_les_fonctions(_).
%
%effacer_les_preselections(S):-
%	retract(choisi(S,_,_)),
%	fail.
%effacer_les_preselections(_).
%
%effacer_les_traits(S):-
%	retract(trait(S,_,_)),
%	fail.
%effacer_les_traits(_).




	
% Pour debugger:


inspect:-
    assert(mode_inspect).
noinspect:-
    eliminer(mode_inspect,0).





inspect(const,Id):-
        mode_inspect,
        const(Id,F,Sem,_),
        nl,nl,write('Je produis le constituant suivant:'),nl,
        write(Id),write(' '),write(F),nl,
        write(Sem),nl.


inspect(preselect,Id):-
        mode_inspect,
        nl,write('Traits preselectionnes: '),nl,
        const(Id,_,_,T),
        write(T),nl.

inspect(selections,Id):-
        mode_inspect,
        nl,write('Traits selectionnes: '),nl,
        const(Id,_,_,T),
        write(T),nl.

inspect(structure,Id):-
        mode_inspect,
        nl,write('Les contraintes de pattern: '),nl,
        lister_patt(Id),nl.

inspect(_,_).





% Pour nettoyer la base de donnees dynamique

clean_all:-
	eliminer(choisi,3),
	eliminer(pattern,3),
	eliminer(domine,3),
        eliminer(const,4),
	eliminer(topic,1),
	eliminer(attente,2),
	eliminer(pattern,2),
	eliminer(temps,2),
        eliminer(cadre_temporel,1).



% Pour afficher des informations internes

lister_fct(Symb):-
    domine(Symb,F,Val),
    write(domine(Symb,F,Val)),
    nl,
    fail.
lister_fct(_).

lister_preselect(X):- nl,
	           write('Voici les preselections effectuees:  '),nl,
                   lister_pre(X).
lister_pre(X):-    domine(X,F,Fsem),
                   choisi(Fsem,S,C),
                   write('Domine:  '),write(F),write('  '),
                   write('Systeme:  '),write(S),write('  '),
                   write('Choix:  '),write(C),write('  '),
		   nl,
                   fail.
lister_pre(_).


lister_patterns(X):- nl,
	          write('Voici les contraintes de pattern:'),nl,
		  lister_patt(X).

lister_patt(X):-  pattern(X,P),
                  write(P),nl,fail.

lister_patt(_):- nl.




% Pour extraire une information de la representation semantique

extraire_trait([Trait === Info |_],Trait,Info).
extraire_trait([_|Reste],Trait,Info):- extraire_trait(Reste,Trait,Info).


extraire_constituant(Id,fonctions,F):-
    const(Id,F,_,_).
extraire_constituant(Id,semantique,S):-
    const(Id,_,S,_).
extraire_constituant(Id,traits,T):-
    const(Id,_,_,T).
