% Librairie de procedures pour la realisation d'un trait.

% On cree une nouvelle fonction tout en lui associant une valeur, c'est
% a-dire un constituant qui devra ulterieurement etre realise.
% Cette valeur est extraite du message ou de la base de connaissances
% Elle fait appel a une fonction "inc" qui depend de la representation
% des connaissances.
% A noter que la fonction est associee a l'unite en cours de traitement.
% Il se peut, aussi, que le constituant ait comme valeur "nil" si on ne
% extraire dans l'environnement l'information lui correspondant.

inclure(Fct):-
        topic(Id),
        const(Id,_,S,_),
        extraire_trait(S,message,M),        
	inc(Fct,S,M,SemFonction),!,
	creer_const([Fct],SemFonction,[],IdFonct),
        asserta(domine(Id,Fct,IdFonct)).

% Inclure un item comme constituant d'une fonction
%inclure(NomF1,F2):-
%        topic(Id),
%         const(Id,_,S,_),
%        inc(F2,S,SemFonction),!,
%        creer_const([Fct],SemFonction,[],IdFonct),
%        fonction(const(Id,Unite),NomF1,F1),
%        asserta(fonction(F1,F2,Symbval)).



% On cree une nouvelle fonction sans lui associer une valeur.
creer(Fonction):-
	topic(Id),
	creer_const([Fonction],[],[],IdFonct),
	asserta(domine(Id,Fonction,IdFonct)).

% On determine de facon aniticipee le trait qui devra etre choisi pour
% un des systemes qui seront parcourus lors de la realisation d'une
% des fonctions de l'unite courante.
% Ceci se fait en ajoutant a le trait a la liste de traits du constituant

% On peut preselectionner un chemin dans le reseau

preselect(Fonction,[Systeme,Trait .. Chemin]):-
        preselect(Fonction,[Systeme,Trait]),
        suite(Systeme,Trait,Sys2),
        preselect(Fonction,[Sys2,Chemin]).
preselect(Fonction,[Systeme,Trait]):-
        Trait \== _ .. _,
        systeme(Systeme,Choix,_),
        member(Trait,Choix),
	topic(X),
	domine(X,Fonction,Id),
        retract(const(Id,F,S,T)),
        assert(const(Id,F,S,[Systeme === Trait | T])).
preselect(_,Chemin):-
        write('erreur: je suis incapable de preselectionner ceci'),nl,
        write(Chemin),nl,
        write('Constituant traite: '),
        topic(Id),write(Id),nl.



% Le cas ou la preselection se fait sur le constituant 
% en cours de production:
preselect([Systeme,Trait]):-
	preselect(constituant,[Systeme,Trait]).

	


% Consulte le lexique afin de determiner la presence d'un 
% trait lexical

trait_lexical(Cat,Item,T):-
	lexique(Cat,Item,_,Traits),
        member(T,Traits).




% On ajoute un nouveau pattern
        
ordre(Pattern):-
	topic(Id),
	assert(pattern(Id,Pattern)).
ordre(F,Pattern):-
        topic(Id),
        domine(Id,F,Fct),
        assert(pattern(Fct,Pattern)).

% On attribue a une fonction toute la structure de sa composante
% semantique
attribuer_semantique(Fonction,Sem):-
        topic(Id),
        domine(Id,Fonction,IdFct),
        retract(const(IdFct,F,_,T)),
        assert(const(IdFct,F,Sem,T)).


% On ajoute un trait dans la composante semantique d'une
% fonction
attribuer(Fonction,TraitSem,Valeur):-
 	topic(Id),
        domine(Id,Fonction,IdFct),
	retract(const(IdFct,F,S,T)),
        % Si le trait existe deja on l'enleve
        enleve(TraitSem===_,S,S1),
	assert(const(IdFct,F,[TraitSem === Valeur|S1],T)).



% Attribue a une fonction (Nouveau) une valeur qui est la meme que
% celle d'une autre fonction deja exitante.
conflater(Fct1,Fct2):-
	topic(X),
	domine(X,Fct1,IdFct1),
        domine(X,Fct2,IdFct2),
        retract(const(IdFct1,F1,S1,T1)),
        retract(const(IdFct2,F2,S2,T2)),
        retract(domine(X,Fct2,IdFct2)),
        asserta(domine(X,Fct2,IdFct1)),
        append(F1,F2,F3),
        append(S1,S2,S3),
        append(T1,T2,T3),
        assert(const(IdFct1,F3,S3,T3)).



% On met dans un terme "lex" la valeur d'une fonction pour signifier 
% qu'elle devra etre lexicalisee a la fin du cycle courant. On indique
% aussi la categorie lexicale de l'item
lexicaliser(F,Cat):-
        topic(X),
        domine(X,F,Id),
        retract(const(Id,ListeF,S,T)),
        assert(const(Id,ListeF,[catlex === Cat | S],T)).

% Indique qu'un constituant doit etre decline
morpho(Fonction):-
        topic(X),
        domine(X,Fonction,Id),
        retract(const(Id,ListeF,S,T)),
        assert(const(Id,ListeF,[decliner===oui | S],T)).    



% On ajoute dans la memoire une information indiquant quel trait a ete
% choisi, pour un systeme et un constituant particuliers.
nouveau_trait(Syst,Trait):-
	topic(N),
	retract(const(N,F,S,T)),
        assert(const(N,F,S,[Syst === Trait|T])).



% trait/2 informe sur le trait choisi dans un systeme pour le constituant 
% en cours de realisation.
trait(Systeme,Trait):-
	topic(Id),
        const(Id,_,_,T),
	extraire_trait(T,Systeme,Trait).


% trait/3 informe sur le trait choisi dans un systeme pour un
% constituant donne
trait(Chemin,Systeme,Trait):-
        val(Chemin,Id),
        const(Id,_,_,T),
        extraire_trait(T,Systeme,Trait).
    
% Verifie si une fonction existe.
existe(Fonction):-
	topic(Id),
	domine(Id,Fonction,IdFct),
	extraire_constituant(IdFct,semantique,Val),
	Val \== [].

% Retourne la valeur semantique du constituant remplissant une fonction.
% Cette fonction appartient a l'unite en cours de realisation.
valeur(Fonction,Val):-
	topic(Id),
	domine(Id,Fonction,IdF),
        extraire_constituant(IdF,semantique,Val).


% Indique si la fonction realisee est Fct
fonction_realisee(Fct):-
	topic(Id),
	const(Id,F,_,_),
	member(Fct,F).




% Retourne la valeur d'une fonction qu'on specifie en donnant le chemin 
% dans l'arbre hierarchique des fonctions.
% On recherchera, en remontant l'arbre hierarchique, la premiere occurrence
% de la premiere fonction indiquee dans le chemin. Puis on redescend en
% parcourant les fonctions indiquees dans le chemin.
% val_const retourne le constituant
% val_sem retourne la semantique associee au constituant
val_const(Chemin,const(Id,F,S,T)):-
	val(Chemin,Id),
        const(Id,F,S,T),!.
val_sem(Chemin,S):-
	val(Chemin,Id),
        const(Id,_,S,_),!.


val(F1 .. Reste,Id):-
        topic(X),!,
        chercher_noeud(X,F1,Origine),
	parcourir_arbre(Origine,F1 .. Reste,Id).
val(Fonction,Id):-
        topic(X),
        chercher_noeud(X,Fonction,Origine),
	domine(Origine,Fonction,Id),!.

% Chercher_noeud retourne le noeud parent d'une fonction 

% La fonction cherchee est un enfant du topic
chercher_noeud(_,racine,racine).
chercher_noeud(Topic,Fonction,Topic):-
	domine(Topic,Fonction,_).
% La fonction cherchee est celle qui est remplie par le topic
% Il faut donc retourner son parent
chercher_noeud(Topic,Fonction,Parent):-
	domine(Parent,Fonction,Topic).
% La fonction recherchee est plus haut dans la hierarchie
chercher_noeud(Topic,Fonction,Const):-
        domine(C,_,Topic),
        C \== Topic,
        chercher_noeud(C,Fonction,Const).



parcourir_arbre(Noeud,Fonction,Id):-
	domine(Noeud,Fonction,Id).
parcourir_arbre(Noeud,F .. R, Id):-
	domine(Noeud,F,Fct),
	parcourir_arbre(Fct,R,Id).

% Retourne la valeur semantique d'un constitant
semantique(X,Sem):- extraire_constituant(X,semantique,Sem).

% Ajoute une fonction ayant comme valeur le constituant courant
% Ne s'utilise que dans des cas tres particuliers
constituant(Fonction):-
	retract(topic(Id)),
        retract(const(Id,F,S,T)),
        assert(topic(const(Id,[Fonction|F],S,T))),
        assert(const(Id,[Fonction|F],S,T)),
	asserta(domine(Id,Fonction,Id)).
